@@ -34,6 +34,7 @@
 int GetFileFormatFromExtension(const std::string &ext, double *version)
 {
   if(ext == ".geo_unrolled")  return FORMAT_GEO;
+  else if(ext == ".py_unrolled") return FORMAT_PY;
   else if(ext == ".auto")     return FORMAT_AUTO;
   else if(ext == ".msh")      return FORMAT_MSH;
   else if(ext == ".msh1")     { if(version) *version = 1.0; return FORMAT_MSH; }
@@ -64,6 +65,7 @@ int GetFileFormatFromExtension(const std::string &ext, double *version)
   else if(ext == ".diff")     return FORMAT_DIFF;
   else if(ext == ".inp")      return FORMAT_INP;
   else if(ext == ".key")      return FORMAT_KEY;
+  else if(ext == ".rad")      return FORMAT_RAD;
   else if(ext == ".celum")    return FORMAT_CELUM;
   else if(ext == ".su2")      return FORMAT_SU2;
   else if(ext == ".nas")      return FORMAT_BDF;
@@ -108,6 +110,7 @@ std::string GetDefaultFileExtension(int format, bool onlyMeshFormats)
   bool mesh = false;
   switch(format){
   case FORMAT_GEO:     name = ".geo_unrolled"; break;
+  case FORMAT_PY:      name = ".py_unrolled"; break;
   case FORMAT_MSH:     name = ".msh"; mesh = true; break;
   case FORMAT_POS:     name = ".pos"; break;
   case FORMAT_X3D:     name = ".x3d"; mesh = true; break;
@@ -129,6 +132,7 @@ std::string GetDefaultFileExtension(int format, bool onlyMeshFormats)
   case FORMAT_DIFF:    name = ".diff"; mesh = true; break;
   case FORMAT_INP:     name = ".inp"; mesh = true; break;
   case FORMAT_KEY:     name = ".key"; mesh = true; break;
+  case FORMAT_RAD:     name = ".rad"; mesh = true; break;
   case FORMAT_CELUM:   name = ".celum"; mesh = true; break;
   case FORMAT_SU2:     name = ".su2"; mesh = true; break;
   case FORMAT_P3D:     name = ".p3d"; mesh = true; break;
@@ -270,7 +274,7 @@ static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
 }
 #endif
 
-void change_print_parameter(int frame)
+static void ChangePrintParameter(int frame)
 {
   double first = CTX::instance()->print.parameterFirst;
   double last = CTX::instance()->print.parameterLast;
@@ -440,6 +444,12 @@ void CreateOutputFile(const std::string &fileName, int format,
        CTX::instance()->mesh.scalingFactor);
     break;
 
+  case FORMAT_RAD:
+    GModel::current()->writeRAD
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.saveGroupsOfNodes,
+       CTX::instance()->mesh.scalingFactor);
+    break;
+
   case FORMAT_CELUM:
     GModel::current()->writeCELUM
       (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
@@ -480,6 +490,11 @@ void CreateOutputFile(const std::string &fileName, int format,
                                 CTX::instance()->print.geoOnlyPhysicals);
     break;
 
+  case FORMAT_PY:
+    GModel::current()->writePY(name, CTX::instance()->print.geoLabels,
+                               CTX::instance()->print.geoOnlyPhysicals);
+    break;
+
   case FORMAT_BREP:
     if(GModel::current()->getOCCInternals())
       GModel::current()->writeOCCBREP(name);
@@ -758,7 +773,7 @@ void CreateOutputFile(const std::string &fileName, int format,
         status_play_manual(!cycle, 0, false);
       for(std::size_t i = 0; i < frames.size(); i++){
         if(cycle == 2)
-          change_print_parameter(i);
+          ChangePrintParameter(i);
         if(fp)
           CreateOutputFile(CTX::instance()->homeDir + frames[i], FORMAT_PPM,
                            false);
