@@ -65,7 +65,6 @@ std::map<std::string, double> Msg::_timers;
 bool Msg::_infoCpu = false;
 bool Msg::_infoMem = false;
 double Msg::_startTime = 0.;
-int Msg::_startMaxThreads = 0;
 int Msg::_warningCount = 0;
 int Msg::_errorCount = 0;
 int Msg::_atLeastOneErrorInRun = 0;
@@ -126,7 +125,6 @@ static void addGmshPathToEnvironmentVar(const std::string &name)
 void Msg::Initialize(int argc, char **argv)
 {
   _startTime = TimeOfDay();
-  _startMaxThreads = GetMaxThreads();
 #if defined(HAVE_MPI)
   int flag;
   MPI_Initialized(&flag);
@@ -216,8 +214,6 @@ void Msg::Finalize()
     MPI_Finalize();
 #endif
   FinalizeOnelab();
-  if(GetMaxThreads() != _startMaxThreads)
-    SetNumThreads(_startMaxThreads);
 }
 
 int Msg::GetCommRank()
@@ -485,9 +481,11 @@ std::string Msg::PrintResources(bool printDate, bool printWallTime,
 
 void Msg::Error(const char *fmt, ...)
 {
-  _errorCount++;
-  _atLeastOneErrorInRun = 1;
-
+#pragma omp critical(MsgError)
+  {
+    _errorCount++;
+    _atLeastOneErrorInRun = 1;
+  }
   char str[5000];
   va_list args;
   va_start(args, fmt);
@@ -495,8 +493,11 @@ void Msg::Error(const char *fmt, ...)
   va_end(args);
   int l = strlen(str); if(str[l - 1] == '\n') str[l - 1] = '\0';
 
-  if(_firstError.empty()) _firstError = str;
-  _lastError = str;
+#pragma omp critical(MsgError)
+  {
+    if(_firstError.empty()) _firstError = str;
+    _lastError = str;
+  }
 
   if(GetVerbosity() >= 1) {
     if(_logFile) fprintf(_logFile, "Error: %s\n", str);
@@ -529,10 +530,10 @@ void Msg::Error(const char *fmt, ...)
 #if defined(HAVE_FLTK)
     if(FlGui::available()) return; // don't throw if GUI is running
 #endif
-    throw _lastError;
+    throw std::runtime_error(_lastError);
   }
   else if(CTX::instance()->abortOnError == 3) {
-    throw _lastError;
+    throw std::runtime_error(_lastError);
   }
   else if(CTX::instance()->abortOnError == 4) {
     Exit(1);
@@ -541,7 +542,10 @@ void Msg::Error(const char *fmt, ...)
 
 void Msg::Warning(const char *fmt, ...)
 {
-  _warningCount++;
+#pragma omp critical(MsgWarning)
+  {
+    _warningCount++;
+  }
 
   if(GetVerbosity() < 2) return;
 
@@ -610,7 +614,7 @@ void Msg::Info(const char *fmt, ...)
 
   if(CTX::instance()->terminal){
     if(_progressMeterCurrent >= 0 && _progressMeterTotal > 1 &&
-       _commSize == 1) {
+        _commSize == 1) {
       int p =  _progressMeterCurrent;
       fprintf(stdout, "Info    : [%3d%%] %s\n", p, str);
     }
@@ -1629,11 +1633,7 @@ int Msg::GetThreadNum(){ return omp_get_thread_num(); }
 #else
 
 int Msg::GetNumThreads(){ return 1; }
-void Msg::SetNumThreads(int num)
-{
-  if(num > 1)
-    Msg::Warning("Setting number of threads to %d requires OpenMP", num);
-}
+void Msg::SetNumThreads(int num){ }
 int Msg::GetMaxThreads(){ return 1; }
 int Msg::GetThreadNum(){ return 0; }
 
