@@ -68,6 +68,7 @@ std::string opt_geometry_double_clicked_curve_command(OPT_ARGS_STR);
 std::string opt_geometry_double_clicked_surface_command(OPT_ARGS_STR);
 std::string opt_geometry_double_clicked_volume_command(OPT_ARGS_STR);
 std::string opt_geometry_occ_target_unit(OPT_ARGS_STR);
+std::string opt_geometry_pipe_default_trihedron(OPT_ARGS_STR);
 std::string opt_solver_socket_name(OPT_ARGS_STR);
 std::string opt_solver_name(OPT_ARGS_STR);
 std::string opt_solver_name0(OPT_ARGS_STR);
@@ -398,6 +399,7 @@ double opt_geometry_surface_type(OPT_ARGS_NUM);
 double opt_geometry_light(OPT_ARGS_NUM);
 double opt_geometry_light_two_side(OPT_ARGS_NUM);
 double opt_geometry_occ_auto_embed(OPT_ARGS_NUM);
+double opt_geometry_occ_safe_unbind(OPT_ARGS_NUM);
 double opt_geometry_occ_auto_fix(OPT_ARGS_NUM);
 double opt_geometry_occ_bounds_use_stl(OPT_ARGS_NUM);
 double opt_geometry_occ_disable_stl(OPT_ARGS_NUM);
@@ -410,6 +412,7 @@ double opt_geometry_occ_union_unify(OPT_ARGS_NUM);
 double opt_geometry_occ_parallel(OPT_ARGS_NUM);
 double opt_geometry_occ_boolean_preserve_numbering(OPT_ARGS_NUM);
 double opt_geometry_occ_scaling(OPT_ARGS_NUM);
+double opt_geometry_occ_export_only_visible(OPT_ARGS_NUM);
 double opt_geometry_occ_import_labels(OPT_ARGS_NUM);
 double opt_geometry_occ_thrusections_degree(OPT_ARGS_NUM);
 double opt_geometry_occ_use_generic_closest_point(OPT_ARGS_NUM);
@@ -471,6 +474,7 @@ double opt_mesh_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_prisms(OPT_ARGS_NUM);
 double opt_mesh_pyramids(OPT_ARGS_NUM);
 double opt_mesh_trihedra(OPT_ARGS_NUM);
+double opt_mesh_transfinite_tri(OPT_ARGS_NUM);
 double opt_mesh_surface_edges(OPT_ARGS_NUM);
 double opt_mesh_surface_faces(OPT_ARGS_NUM);
 double opt_mesh_volume_edges(OPT_ARGS_NUM);
@@ -525,6 +529,7 @@ double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
 double opt_mesh_stl_remove_duplicate_triangles(OPT_ARGS_NUM);
 double opt_mesh_stl_one_solid_per_surface(OPT_ARGS_NUM);
 double opt_mesh_stl_linear_deflection(OPT_ARGS_NUM);
+double opt_mesh_stl_linear_deflection_relative(OPT_ARGS_NUM);
 double opt_mesh_stl_angular_deflection(OPT_ARGS_NUM);
 double opt_mesh_boundary_layer_fan_elements(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
@@ -535,6 +540,8 @@ double opt_mesh_algo_recombine(OPT_ARGS_NUM);
 double opt_mesh_max_retries(OPT_ARGS_NUM);
 double opt_mesh_recombine_all(OPT_ARGS_NUM);
 double opt_mesh_recombine_optimize_topology(OPT_ARGS_NUM);
+double opt_mesh_recombine_node_repositioning(OPT_ARGS_NUM);
+double opt_mesh_recombine_minimum_quality(OPT_ARGS_NUM);
 double opt_mesh_recombine3d_all(OPT_ARGS_NUM);
 double opt_mesh_recombine3d_level(OPT_ARGS_NUM);
 double opt_mesh_recombine3d_conformity(OPT_ARGS_NUM);
@@ -542,6 +549,7 @@ double opt_mesh_flexible_transfinite(OPT_ARGS_NUM);
 double opt_mesh_algo_subdivide(OPT_ARGS_NUM);
 double opt_mesh_mesh_only_visible(OPT_ARGS_NUM);
 double opt_mesh_mesh_only_empty(OPT_ARGS_NUM);
+double opt_mesh_min_line_nodes(OPT_ARGS_NUM);
 double opt_mesh_min_circle_nodes(OPT_ARGS_NUM);
 double opt_mesh_allow_swap_edge_angle(OPT_ARGS_NUM);
 double opt_mesh_min_curve_nodes(OPT_ARGS_NUM);
@@ -562,6 +570,7 @@ double opt_mesh_ho_curve_outer_BL(OPT_ARGS_NUM);
 double opt_mesh_ho_max_rho(OPT_ARGS_NUM);
 double opt_mesh_ho_max_angle(OPT_ARGS_NUM);
 double opt_mesh_ho_max_in_angle(OPT_ARGS_NUM);
+double opt_mesh_ho_fix_bnd_nodes(OPT_ARGS_NUM);
 double opt_mesh_second_order_linear(OPT_ARGS_NUM);
 double opt_mesh_second_order_incomplete(OPT_ARGS_NUM);
 double opt_mesh_cgns_import_order(OPT_ARGS_NUM);
@@ -576,6 +585,7 @@ double opt_mesh_draw_skin_only(OPT_ARGS_NUM);
 double opt_mesh_save_all(OPT_ARGS_NUM);
 double opt_mesh_save_element_tag_type(OPT_ARGS_NUM);
 double opt_mesh_save_parametric(OPT_ARGS_NUM);
+double opt_mesh_save_without_orphans(OPT_ARGS_NUM);
 double opt_mesh_save_topology(OPT_ARGS_NUM);
 double opt_mesh_read_groups_of_elements(OPT_ARGS_NUM);
 double opt_mesh_save_groups_of_elements(OPT_ARGS_NUM);
