@@ -21,13 +21,13 @@
 #include "discreteEdge.h"
 #include "discreteFace.h"
 #include "ExtrudeParams.h"
-#include "Field.h"
 
 #if defined(HAVE_MESH)
 #include "meshGFace.h"
 #include "meshGFaceOptimize.h"
 #include "BackgroundMeshTools.h"
 #include "meshGFaceBipartiteLabelling.h"
+#include "Field.h"
 #endif
 
 #if defined(HAVE_ALGLIB)
@@ -75,6 +75,13 @@ int GFace::getMeshSizeFromBoundary() const
     return CTX::instance()->mesh.lcExtendFromBoundary;
 }
 
+bool GFace::isOrphan()
+{
+  if(model()->getNumRegions())
+    return regions().empty();
+  return false;
+}
+
 int GFace::delEdge(GEdge *edge)
 {
   const auto found = std::find(l_edges.begin(), l_edges.end(), edge);
@@ -237,6 +244,7 @@ void GFace::resetMeshAttributes()
   meshAttributes.meshSizeFactor = 1.;
   meshAttributes.algorithm = 0;
   meshAttributes.meshSizeFromBoundary = -1;
+  meshAttributes.transfinite3 = false;
 }
 
 SBoundingBox3d GFace::bounds(bool fast)
@@ -347,7 +355,7 @@ std::vector<MVertex *> GFace::getEmbeddedMeshVertices(bool force) const
 
 std::vector<GVertex *> GFace::vertices() const
 {
-  std::set<GVertex *> v;
+  std::set<GVertex *, GEntityPtrLessThan> v;
   for(auto ge : l_edges) {
     GVertex *const v1 = ge->getBeginVertex();
     if(v1) v.insert(v1);
@@ -486,7 +494,7 @@ void GFace::writeGEO(FILE *fp)
       fprintf(fp, "Surface(%d) = {%d};\n", tag(), tag());
     }
     else {
-      Msg::Error("Skipping surface %d in export", tag());
+      Msg::Warning("Skipping surface %d in export", tag());
     }
   }
 
@@ -514,6 +522,46 @@ void GFace::writeGEO(FILE *fp)
   if(meshAttributes.reverseMesh) fprintf(fp, "Reverse Surface {%d};\n", tag());
 }
 
+void GFace::writePY(FILE *fp)
+{
+  // This is by no means complete - merely a placeholder for a future
+  // implementation
+
+  if(geomType() == DiscreteSurface || geomType() == BoundaryLayerSurface) return;
+
+  const char *factory = getNativeType() == OpenCascadeModel ? "occ" : "geo";
+
+  std::size_t numcl = 0;
+  for(std::size_t i = 0; i < edgeLoops.size(); i++) {
+    std::vector<GEdge *> edges;
+    std::vector<int> signs;
+    edgeLoops[i].getEdges(edges);
+    edgeLoops[i].getSigns(signs);
+    if(edges.size() && edges.size() == signs.size()) {
+      fprintf(fp, "s%d_cl%lu = gmsh.model.%s.addCurveLoop([", tag(), ++numcl,
+              factory);
+      for(std::size_t j = 0; j < edges.size(); j++) {
+        if(j) fprintf(fp, ", ");
+        fprintf(fp, "%d", edges[j]->tag() * signs[j]);
+      }
+      fprintf(fp, "])\n");
+    }
+  }
+
+  if(geomType() == GEntity::Plane || geomType() == GEntity::ParametricSurface) {
+    fprintf(fp, "gmsh.model.%s.addPlaneSurface([", factory);
+    for(std::size_t i = 0; i < numcl; i++) {
+      if(i) fprintf(fp, ", ");
+      fprintf(fp, "s%d_cl%lu", tag(), i + 1);
+    }
+    fprintf(fp, "], %d)\n", tag());
+  }
+  else {
+    // TODO
+    Msg::Warning("Skipping surface %d in export", tag());
+  }
+}
+
 void GFace::computeMeanPlane()
 {
   std::vector<SPoint3> pts;
@@ -1084,9 +1132,9 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
   if(!onSurface) return;
 
   if(relax < 1.e-3)
-    Msg::Error("Inverse surface mapping could not converge");
+    Msg::Warning("Inverse surface mapping could not converge");
   else {
-    Msg::Info("point %g %g %g : Relaxation factor = %g", X, Y, Z, 0.75 * relax);
+    Msg::Info("Point %g %g %g: Relaxation factor = %g", X, Y, Z, 0.75 * relax);
     XYZtoUV(X, Y, Z, U, V, 0.75 * relax, onSurface, convTestXYZ);
   }
 }
@@ -1328,7 +1376,7 @@ bool GFace::buildRepresentationCross(bool force)
           cross[dir].push_back(std::vector<SPoint3>());
       }
     }
-    while(cross[dir].back().empty())
+    while(!cross[dir].empty() && cross[dir].back().empty())
       cross[dir].pop_back();
   }
 
@@ -1510,6 +1558,36 @@ bool GFace::fillVertexArray(bool force)
   return true;
 }
 
+bool GFace::storeSTLAsMesh()
+{
+  // as the STL might be non-conforming, we make no effort to have a conformal
+  // mesh - nodes will not be classified on boundaries, and not shared with
+  // adjacent entities
+  if(stl_vertices_xyz.size()) {
+    for(std::size_t i = 0; i < stl_vertices_xyz.size(); i++) {
+      SPoint3 &p(stl_vertices_xyz[i]);
+      mesh_vertices.push_back(new MVertex(p.x(), p.y(), p.z(), this));
+    }
+  }
+  else if(stl_vertices_uv.size()) {
+    for(std::size_t i = 0; i < stl_vertices_uv.size(); i++) {
+      SPoint2 &p(stl_vertices_uv[i]);
+      GPoint gp = point(p);
+      mesh_vertices.push_back(new MFaceVertex(gp.x(), gp.y(), gp.z(),
+                                              this, p.x(), p.y()));
+    }
+  }
+  else {
+    return false;
+  }
+  for(std::size_t i = 0; i < stl_triangles.size(); i += 3) {
+    triangles.push_back(new MTriangle(mesh_vertices[stl_triangles[i]],
+                                      mesh_vertices[stl_triangles[i + 1]],
+                                      mesh_vertices[stl_triangles[i + 2]]));
+  }
+  return true;
+}
+
 int GFace::genusGeom() const
 {
   int nSeams = 0;
@@ -1716,7 +1794,7 @@ static void meshCompound(GFace *gf, bool verbose)
 
   std::set<GEdge *, GEntityPtrLessThan> bnd, emb1;
   std::set<GVertex *, GEntityPtrLessThan> emb0;
-  std::vector<int> phys;
+  std::set<int> phys;
   for(std::size_t i = 0; i < gf->compound.size(); i++) {
     auto *c = (GFace *)gf->compound[i];
     df->triangles.insert(df->triangles.end(), c->triangles.begin(),
@@ -1747,7 +1825,7 @@ static void meshCompound(GFace *gf, bool verbose)
     }
     c->compoundSurface = df;
     if(!magic) {
-      phys.insert(phys.end(), c->physicals.begin(), c->physicals.end());
+      phys.insert(c->physicals.begin(), c->physicals.end());
       c->physicals.clear();
     }
   }
@@ -1773,7 +1851,7 @@ static void meshCompound(GFace *gf, bool verbose)
   Field *backgroundField = fields->get(BGTAG);
 
   if(df->createGeometry()) {
-    Msg::Error("Could not create geometry of discrete face %d (check "
+    Msg::Error("Could not create geometry of discrete surface %d (check "
                "orientation of input triangulations)",
                df->tag());
   }
@@ -1798,7 +1876,8 @@ static void meshCompound(GFace *gf, bool verbose)
   }
 
   if(!magic) {
-    df->physicals = phys;
+    df->physicals.clear();
+    df->physicals.insert(df->physicals.end(), phys.begin(), phys.end());
     return;
   }
 
@@ -2059,21 +2138,35 @@ void GFace::setMeshMaster(GFace *master, const std::vector<double> &tfo)
       sign = -1;
     }
 
-    // multiple matches (when several curves have the same begin/end points)
+    // if there are multiple matches (when several curves have the same
+    // begin/end points), we must compare the geometry: mid-points (general) or
+    // transformed bounding boxes as a fallback (ok for translations or
+    // rotations by n*pi/2)
     SBoundingBox3d localbb = localEdge->bounds(true);
-    // using the global geometrical tolerance does not work well, as the
-    // accuracy of bounding boxes can be poor (e.g. in OCC, computed through a
-    // mesh, ...)
     double tol = localbb.diag() * 1e-3;
-
+    Range<double> localr = localEdge->parBounds(0);
+    GPoint localp = localEdge->point(0.5 * (localr.low() + localr.high()));
     if(!masterEdge && numf) {
       auto ret = m_vtxToEdge.equal_range(forward);
       for(auto it = ret.first; it != ret.second; it++) {
-        SBoundingBox3d masterbb = it->second->bounds(true);
+        GEdge *ge = it->second;
+        Range<double> masterr = ge->parBounds(0);
+        GPoint masterp = ge->point(0.5 * (masterr.low() + masterr.high()));
+        if(localp.succeeded() && masterp.succeeded()) {
+          SPoint3 p1(localp.x(), localp.y(), localp.z());
+          SPoint3 p2(masterp.x(), masterp.y(), masterp.z());
+          p2.transform(tfo);
+          if(p1.distance(p2) < tol) {
+            masterEdge = ge;
+            sign = 1;
+            break;
+          }
+        }
+        SBoundingBox3d masterbb = ge->bounds(true);
         masterbb.transform(tfo);
         if(masterbb.min().distance(localbb.min()) < tol &&
            masterbb.max().distance(localbb.max()) < tol) {
-          masterEdge = it->second;
+          masterEdge = ge;
           sign = 1;
           break;
         }
@@ -2082,11 +2175,24 @@ void GFace::setMeshMaster(GFace *master, const std::vector<double> &tfo)
     if(!masterEdge && numb) {
       auto ret = m_vtxToEdge.equal_range(backward);
       for(auto it = ret.first; it != ret.second; it++) {
-        SBoundingBox3d masterbb = it->second->bounds(true);
+        GEdge *ge = it->second;
+        Range<double> masterr = ge->parBounds(0);
+        GPoint masterp = ge->point(0.5 * (masterr.low() + masterr.high()));
+        if(localp.succeeded() && masterp.succeeded()) {
+          SPoint3 p1(localp.x(), localp.y(), localp.z());
+          SPoint3 p2(masterp.x(), masterp.y(), masterp.z());
+          p2.transform(tfo);
+          if(p1.distance(p2) < tol) {
+            masterEdge = ge;
+            sign = -1;
+            break;
+          }
+        }
+        SBoundingBox3d masterbb = ge->bounds(true);
         masterbb.transform(tfo);
         if(masterbb.min().distance(localbb.min()) < tol &&
            masterbb.max().distance(localbb.max()) < tol) {
-          masterEdge = it->second;
+          masterEdge = ge;
           sign = -1;
           break;
         }
@@ -2094,7 +2200,7 @@ void GFace::setMeshMaster(GFace *master, const std::vector<double> &tfo)
     }
 
     if(!masterEdge) {
-      Msg::Error("Could not find counterpart of curve %d with end points %d-%d "
+      Msg::Error("Could not find counterpart of curve %d with end points %d %d "
                  "(corresponding to curve with end points %d %d) in surface %d",
                  localEdge->tag(), lPair.first->tag(), lPair.second->tag(),
                  forward.first->tag(), forward.second->tag(), master->tag());
@@ -2343,7 +2449,7 @@ void GFace::setMeshMaster(GFace *master, const std::map<int, int> &edgeCopies)
     }
     else {
       Msg::Error("Only rotations or translations can currently be computed "
-                 "automatically for periodic faces: face %d not meshed",
+                 "automatically for periodic surfaces: surface %d not meshed",
                  tag());
       return;
     }
@@ -2361,7 +2467,8 @@ void GFace::addElement(int type, MElement *e)
   case TYPE_TRI: addTriangle(reinterpret_cast<MTriangle *>(e)); break;
   case TYPE_QUA: addQuadrangle(reinterpret_cast<MQuadrangle *>(e)); break;
   case TYPE_POLYG: addPolygon(reinterpret_cast<MPolygon *>(e)); break;
-  default: Msg::Error("Trying to add unsupported element in face");
+  default:
+    Msg::Error("Trying to add unsupported element in surface %d", tag());
   }
 }
 
@@ -2383,7 +2490,19 @@ void GFace::removeElement(int type, MElement *e)
                         reinterpret_cast<MPolygon *>(e));
     if(it != polygons.end()) polygons.erase(it);
   } break;
-  default: Msg::Error("Trying to remove unsupported element in face");
+  default:
+    Msg::Error("Trying to remove unsupported element in surface %d", tag());
+  }
+}
+
+void GFace::removeElements(int type)
+{
+  switch(type) {
+  case TYPE_TRI: triangles.clear(); break;
+  case TYPE_QUA: quadrangles.clear(); break;
+  case TYPE_POLYG: polygons.clear(); break;
+  default:
+    Msg::Error("Trying to remove unsupported elements in surface %d", tag());
   }
 }
 
