@@ -75,7 +75,8 @@ private:
   int _readMSH4(const std::string &name);
   int _writeMSH4(const std::string &name, double version, bool binary,
                  bool saveAll, bool saveParametric, double scalingFactor,
-                 bool append, int partitionToSave = 0);
+                 bool append, int partitionToSave = 0,
+                 std::map<GEntity*, SBoundingBox3d> *entityBounds = nullptr);
   int _writePartitionedMSH4(const std::string &baseName, double version,
                             bool binary, bool saveAll, bool saveParametric,
                             double scalingFactor);
@@ -151,6 +152,9 @@ protected:
   // the set of all used mesh partition numbers
   std::size_t _numPartitions;
 
+  // additional attributes (e.g. stored in extra sections of MSH files)
+  std::map<std::string, std::vector<std::string> > _attributes;
+
 protected:
   // store the elements given in the map (indexed by elementary region
   // number) into the model, creating discrete geometrical entities on
@@ -621,7 +625,13 @@ public:
   void checkMeshCoherence(double tolerance);
 
   // remove duplicate mesh vertices
-  int removeDuplicateMeshVertices(double tolerance);
+  int removeDuplicateMeshVertices(double tolerance,
+                                  const std::vector<GEntity*> &entities =
+                                  std::vector<GEntity*>());
+
+  // remove duplicate mesh elements (within an entity)
+  int removeDuplicateMeshElements(const std::vector<GEntity*> &entities =
+                                  std::vector<GEntity*>());
 
   // create a geometry (i.e. a parametrization for curves and surfaces) for the
   // given discrete entities (or all of them if dimTags is empty)
@@ -663,7 +673,7 @@ public:
   // make the mesh a high order mesh at order N (linear is 1 if the high order
   // points are not placed on the geometry of the model; incomplete is 1 if
   // incomplete basis are used)
-  int setOrderN(int order, int linear, int incomplete);
+  int setOrderN(int order, int linear, int incomplete, int onlyVisible);
 
   // refine the mesh by splitting all elements
   int refineMesh(int linear, bool splitIntoQuads = false,
@@ -696,7 +706,8 @@ public:
                           const std::vector<int> &domain,
                           const std::vector<int> &subdomain,
                           const std::vector<int> &dim);
-  void computeHomology();
+  void clearHomologyRequests();
+  void computeHomology(std::vector<std::pair<int, int> > &newPhysicals);
 
   // mesh size callback
   std::function<double(int, int, double, double, double, double)> lcCallback;
@@ -710,17 +721,32 @@ public:
     return _curvatures;
   }
 
+  // reverse engineering of extruded shapes
+  bool addAutomaticExtrusionConstraints(const std::vector<int> &numElements,
+                                        const std::vector<double> &heights,
+                                        const bool recombine,
+                                        const std::vector<int> &regionTag);
+
+  // get additional attributes
+  std::map<std::string, std::vector<std::string> > &getAttributes()
+  {
+    return _attributes;
+  }
+
   // "automatic" IO based on Gmsh global functions
   void load(const std::string &fileName);
   void save(const std::string &fileName);
 
-  // Gmsh native CAD format (readGEO is static, since it can create
-  // multiple models)
+  // GEO file interface (readGEO is static, since it can create multiple models)
   static int readGEO(const std::string &name);
   int writeGEO(const std::string &name, bool printLabels = true,
                bool onlyPhysicals = false);
   int exportDiscreteGEOInternals();
 
+  // PY export (mostly a placeholder for now)
+  int writePY(const std::string &name, bool printLabels = true,
+              bool onlyPhysicals = false);
+
   // OCC model
   int readOCCBREP(const std::string &name);
   int readOCCSTEP(const std::string &name);
@@ -861,6 +887,10 @@ public:
   int writeKEY(const std::string &name, int saveAll = 0,
                int saveGroupsOfNodes = 0, double scalingFactor = 1.0);
 
+  // RADIOSS
+  int writeRAD(const std::string &name, int saveAll = 0,
+               int saveGroupsOfNodes = 0, double scalingFactor = 1.0);
+
   // CELUM
   int writeCELUM(const std::string &name, bool saveAll = false,
                  double scalingFactor = 1.0);
