@@ -620,8 +620,8 @@ static void copy_periodicity(std::vector<Pair<GEType *, GEType *> > &eCor,
       GEType *newSrc = tgtIter->second;
       newTgt->setMeshMaster(newSrc, oldTgt->affineTransform);
 
-      std::map<MVertex *, MVertex *, MVertexPtrLessThan> &oldV2v = oldTgt->correspondingVertices;
-      std::map<MVertex *, MVertex *, MVertexPtrLessThan> &newV2v = newTgt->correspondingVertices;
+      std::map<MVertex *, MVertex *> &oldV2v = oldTgt->correspondingVertices;
+      std::map<MVertex *, MVertex *> &newV2v = newTgt->correspondingVertices;
 
       auto vIter = oldV2v.begin();
       for(; vIter != oldV2v.end(); ++vIter) {
