@@ -5,7 +5,6 @@
 
 #include <string.h>
 #include <sstream>
-#include "Field.h"
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GModel.h"
@@ -20,6 +19,10 @@
 #include "OS.h"
 #include "Parser.h"
 
+#if defined(HAVE_MESH)
+#include "Field.h"
+#endif
+
 #if defined(HAVE_ONELAB)
 #include "onelab.h"
 #endif
@@ -372,12 +375,16 @@ void scriptSetTransfiniteVolume(std::vector<int> &l,
   for(auto &lang : CTX::instance()->scriptLang) {
     std::ostringstream sstream;
     if(lang == "geo") {
-      sstream << "Transfinite Volume{" << l[0] << "} = {";
-      for(std::size_t i = 1; i < l.size(); i++) {
-        if(i > 1) sstream << ", ";
-        sstream << l[i];
+      sstream << "Transfinite Volume{" << l[0] << "}";
+      if(l.size() > 1) {
+        sstream << " = {";
+        for(std::size_t i = 1; i < l.size(); i++) {
+          if(i > 1) sstream << ", ";
+          sstream << l[i];
+        }
+        sstream << "}";
       }
-      sstream << "};";
+      sstream << ";";
     }
     else {
       // TODO
@@ -459,6 +466,7 @@ void scriptAddFieldOption(int field_id, const std::string &option_name,
                           const std::string &option_value, int option_type,
                           const std::string &fileName)
 {
+#if defined(HAVE_MESH)
   for(auto &lang : CTX::instance()->scriptLang) {
     std::ostringstream sstream;
     if(lang == "geo") {
@@ -493,6 +501,7 @@ void scriptAddFieldOption(int field_id, const std::string &option_name,
     }
     scriptAddCommand(sstream.str(), fileName, lang);
   }
+#endif
 }
 
 void scriptAddField(int field_id, const std::string &type_name,
